@@ -27,7 +27,7 @@ Glib::RefPtr<Glib::MainLoop> mainloop;
 // This function is used to receive asynchronous messages in the main loop.
 bool on_bus_message(const Glib::RefPtr<Gst::Bus>& /* bus */, const Glib::RefPtr<Gst::Message>& message)
 {
-  switch (message->get_message_type()) {
+  switch(message->get_message_type()) {
     case Gst::MESSAGE_EOS:
       std::cout << "End of stream." << std::endl;
       mainloop->quit();
@@ -61,6 +61,8 @@ void on_handoff(const Glib::RefPtr<Gst::Buffer>& buffer,
                 const Glib::RefPtr<Gst::Pad>& pad)
 {
   std::cout << "Gst::FakeSrc's handoff signal triggered." << std::endl;
+  if(buffer->flag_is_set(Gst::BUFFER_FLAG_DISCONT))
+    std::cout << "The Gst::FakeSrc buffer's discont flag is set." << std::endl;
 }
 
 int main(int argc, char* argv[])
@@ -74,15 +76,18 @@ int main(int argc, char* argv[])
   Glib::RefPtr<Gst::FakeSrc> source = Gst::FakeSrc::create();
   Glib::RefPtr<Gst::FakeSink> sink = Gst::FakeSink::create();
 
-  if (!pipeline || !source || !sink) {
+  if(!pipeline || !source || !sink) {
     std::cerr << "One element could not be created" << std::endl;
     return -1;
   }
 
   // Put all elements in a bin:
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
+#endif
     pipeline->add(source)->add(sink);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   }
   catch(const Glib::Error& ex)
   {
@@ -91,6 +96,7 @@ int main(int argc, char* argv[])
 
     return -1;
   }
+#endif
 
   // Link together:
   source->link(sink);
@@ -99,10 +105,18 @@ int main(int argc, char* argv[])
   pipeline->get_bus()->add_watch(sigc::ptr_fun(&on_bus_message));
 
   // Set number of buffers fakesink creates to low number:
+#ifdef GLIBMM_PROPERTIES_ENABLED
   source->property_num_buffers() = 5;
+#else
+  source->set_property("num_buffers", 5);
+#endif
 
   // Enable the fakesink handoff signal emition and connect slot:
+#ifdef GLIBMM_PROPERTIES_ENABLED
   sink->property_signal_handoffs() = true;
+#else
+  sink->set_property("signal_handoffs", true);
+#endif
   sink->signal_handoff().connect(sigc::ptr_fun(on_handoff));
 
   // Now set to playing and iterate:
